/*
 * Copyright (c) 2013 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global $, tizen, window, document, history, console*/
// jshint latedef: nofunc

/**
 * Service application id.
 *
 * @type {string}
 */
var gServiceAppId = 'org.example.hybridservice',

    /**
     * Service port name.
     *
     * @type {string}
     */
    gServicePortName = 'SAMPLE_PORT',

    /**
     * Local message port name.
     *
     * @type {string}
     */
    gLocalMessagePortName = 'SAMPLE_PORT_REPLY',

    /**
     * Local message port object.
     *
     * @type {LocalMessagePort}
     */
    gLocalMessagePort = null,

    /**
     * Remote message port object.
     *
     * @type {RemoteMessagePort}
     */
    gRemoteMessagePort = null,

    /**
     * Local message port listener id that can be used later
     * to remove the listener.
     *
     * @type {number}
     */
    gLocalMessagePortWatchId = null,

    /**
     * Indicates whether the message port is during start-up.
     *
     * @type {boolean}
     */
    isStarting = false;

/**
 * Writes a message on the screen.
 *
 * @param {string} message
 */
function writeToScreen(message) {
    'use strict';

    var today = new Date(),
        time = today.getFullYear() + '-' + (today.getMonth() + 1) + '-' +
            today.getDate() + ' ' + today.getHours() + ':' +
            today.getMinutes() + ':' + today.getSeconds() + '.' +
            today.getMilliseconds(),
        str = '<li class="ui-li-static">' + message +
            '<div class="li-text-sub">' + time + '</div></li>';

    $('#logs').append(str).listview('refresh');
    document.getElementById('logs').scrollIntoView(false);
}

/**
 * Sends message to another application.
 *
 * @param {string} command
 */
function sendCommand(command) {
    'use strict';

    try {
        gRemoteMessagePort.sendMessage([{key: 'command', value: command}],
            gLocalMessagePort);
        writeToScreen('Sending: ' + command);
    } catch (error) {
        console.error(error);
    }
}

/**
 * Performs action after receiving message from another application.
 *
 * @param {MessagePortDataItem[]} data
 */
function onReceive(data) {
    'use strict';

    var message = null,
        i = 0,
        len = data.length;

    for (i = 0; i < len; i += 1) {
        if (data[i].key === 'server') {
            message = data[i].value;
        }
    }

    writeToScreen('Received : ' + message);

    if (message === 'WELCOME') {
        sendCommand('start');
    } else if (message === 'stopped') {
        sendCommand('exit');
    } else if (message === 'exit') {
        if (gRemoteMessagePort) {
            gRemoteMessagePort = null;
        }
        if (gLocalMessagePort) {
            try {
                gLocalMessagePort
                    .removeMessagePortListener(gLocalMessagePortWatchId);
                gLocalMessagePort = null;
            } catch (error) {
                console.error(error);
            }
        }
    }
}

/**
 * Initializes message port.
 */
function startMessagePort() {
    'use strict';

    try {
        gLocalMessagePort = tizen.messageport
            .requestLocalMessagePort(gLocalMessagePortName);
        gLocalMessagePortWatchId = gLocalMessagePort
            .addMessagePortListener(function onDataReceive(data, remote) {
                onReceive(data, remote);
            });
    } catch (e) {
        gLocalMessagePort = null;
        writeToScreen(e.name);
    }

    try {
        gRemoteMessagePort = tizen.messageport
            .requestRemoteMessagePort(gServiceAppId, gServicePortName);
    } catch (ex) {
        gRemoteMessagePort = null;
        writeToScreen(ex.name);
    }

    isStarting = false;

    sendCommand('connect');
}

/**
 * Displays popup with alert message.
 *
 * @param {string} message
 */
function showAlert(message) {
    'use strict';

    var alertPopup = $('#alert-popup');

    alertPopup.find('#message').html(message);
    alertPopup.popup('open', {positionTo: 'window'});
}

/**
 * Launches hybrid service application.
 */
function launchServiceApp() {
    'use strict';

    function onSuccess() {
        start();
    }

    function onError(err) {
        console.error('Service Applaunch failed', err);
        isStarting = false;
        showAlert('Failed to launch HybridServiceApp!');
    }

    try {
        tizen.application.launch(gServiceAppId, onSuccess, onError);
    } catch (exc) {
        console.error('Exception while launching HybridServiceApp: ' +
            exc.message);
        showAlert('Exception while launching HybridServiceApp:<br>' +
            exc.message);
    }
}

/**
 * Performs action on a list of application contexts
 * for applications that are currently running on a device.
 *
 * @param {ApplicationContext[]} contexts
 */
function onGetAppsContextSuccess(contexts) {
    'use strict';

    var i = 0,
        len = contexts.length,
        appInfo = null;

    for (i = 0; i < len; i = i + 1) {
        try {
            appInfo = tizen.application.getAppInfo(contexts[i].appId);
        } catch (exc) {
            showAlert('Exception while getting application info:<br>' +
                exc.message);
        }

        if (appInfo.id === gServiceAppId) {
            break;
        }
    }

    if (i >= len) {
        launchServiceApp();
    } else {
        startMessagePort();
    }
}

/**
 * Performs action when getAppsContext method of tizen.application API
 * results in error.
 *
 * @param {Error} err
 */
function onGetAppsContextError(err) {
    'use strict';

    console.error('getAppsContext exc', err);
}

/**
 * Starts obtaining information about applications
 * that are currently running on a device.
 */
function start() {
    'use strict';

    try {
        tizen.application.getAppsContext(onGetAppsContextSuccess,
            onGetAppsContextError);
    } catch (e) {
        writeToScreen('Get AppContext Error: ' + e.message);
    }
}

$(document).delegate('#main', 'pageinit', function onMainPageInit() {
    'use strict';

    $('#btn-start').on('click', function onStartBtnTap() {
        if (gLocalMessagePort) {
            showAlert('Cannot start:<br>already running');
        } else if (isStarting) {
            showAlert('Cannot start:<br>service is starting');
        } else {
            isStarting = true;
            start();
        }
        return false;
    });

    $('#btn-stop').on('click', function onStopBtnTap() {
        if (isStarting) {
            showAlert('Cannot stop:<br>service is starting');
        } else if (gRemoteMessagePort) {
            sendCommand('stop');
        } else {
            showAlert('Cannot stop:<br>not running');
        }
        return false;
    });

    $('#btn-clear').on('click', function onClearBtnTap() {
        $('#logs').empty().listview('refresh');
        return false;
    });

    $(window).on('tizenhwkey', function onTizenHWKey(e) {
        if (e.originalEvent.keyName === 'back') {
            if ($.mobile.activePage.attr('id') === 'main') {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (exc) {
                    console.error('Error: ', exc.message);
                }
            } else {
                history.back();
            }
        }
    });
});
